#include "avrdude.h"
#include "programmer.h"
#include "part.h"
#include <string.h>
#include <stdarg.h>

extern "C" {
#if defined(__WINDOWS__)
#include "avrdude/libavrdude.h"
#else
#include "libavrdude.h"
#endif

/*
#include "avrdude/avrdude.h"
#include "avrdude/avr.h"
#include "avrdude/config.h"
#include "avrdude/confwin.h"
#include "avrdude/fileio.h"
#include "avrdude/lists.h"
#include "avrdude/par.h"
#include "avrdude/pindefs.h"
#include "avrdude/term.h"
#include "avrdude/safemode.h"
#include "avrdude/update.h"
*/

// dummy:
void crcappend() {}
void crcverify() {}

const char * version      = "1.0";
char * progname;
int    verbose;     /* verbose output */
int    ovsigck;     /* 1=override sig check, 0=don't */
int    quell_progress; /* un-verebose output */
int    do_cycles;   /* track erase-rewrite cycles */

char   progbuf[PATH_MAX]; /* temporary buffer of spaces the same
                             length as progname; used for lining up
                             multiline messages */
}

void doAvrdudeProgress(int percent, double etime);

static void do_update_progress(int percent, double etime, char *hdr) {
  doAvrdudeProgress(percent, etime);
}


Avrdude::Avrdude(const std::string & appName, const std::string & configFile) {
  ::verbose = 5;
  ::quell_progress = 0;
  ::strcpy(progbuf, appName.c_str());
  progname = progbuf;
  ::init_config();
  ::read_config(configFile.c_str());
  ::update_progress = do_update_progress;
}

Avrdude::~Avrdude() {
}

Programmer * Avrdude::locateProgrammer(const std::string & name) {
  return Programmer::locateProgrammer(name);
}

Part * Avrdude::locatePart(const std::string & name) {
  return Part::locatePart(name);
}

void doAvrdudeMessage(int type, const char * msg);

extern "C" {
  int avrdude_message(const int msglvl, const char *format, ...) {
    char msg[4096];
    va_list aptr;
    va_start(aptr, format);
    vsnprintf(msg, sizeof(msg), format, aptr);
    va_end(aptr);
    doAvrdudeMessage(msglvl, msg);
    return 0;
  }
}
